package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.io.File;

public class AppletAction extends Action {
    

    public String process (Element message) {


	// find the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "applet.xsl");

	if (stylesheet==null) {
	    return GSHTML.errorPage("applet stylesheet not found!");
	}

	Element request = (Element)message.getElementsByTagName(GSXML.REQUEST_ELEM).item(0);

	// subaction is display/request
	String request_type = request.getAttribute(GSXML.SUBACTION_ATT); // should be 'd' or 'r'
	if (!request_type.equals("d")&&!request_type.equals("r")) {
	    return GSHTML.errorPage("the sa arg to a=a should be d or r!!");
	}
	
	// get the collection and service param
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	cgi_.toLong(cgi_paramList);
	HashMap params = GSXML.extractParams(cgi_paramList);

	String collection = (String)params.get("collection");
	String service_name=(String)params.get("serviceName");
	service_name+= "Applet";
	
	String to=null;
	if (collection==null||collection.equals("") ) {
	    to = service_name;
	} else {
	    to = GSPath.appendLink(collection, service_name);
	}

	System.out.println("to="+to);


	if (request_type.equals("d")) {
	    // we are just displaying the applet - get the description from
	    // the service, and process using xslt

	    //build up the mr request
	
	    Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    mr_message.appendChild(mr_request);
	    mr_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	    mr_request.setAttribute(GSXML.TO_ATT, to);
	    //mr_request.setAttribute(GSXML."info", "appletInfo");
	    
	    Element mr_response = (Element)mr_.process(mr_message);
	    
	    System.out.println("applet mr response =");
	    System.out.println(converter_.getString(mr_response));
	    // create the return page tree
	    Element page = doc_.createElement(GSXML.PAGE_ELEM);
	    page.setAttribute(GSXML.LANG_ATT, message.getAttribute(GSXML.LANG_ATT));
	    // add the lang stuff from message
	    page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.TRANSLATION_ELEM), true));
	    // add the config stuff from message
	    page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));
	    
	    page.appendChild(doc_.importNode(request, true));

	    // add in the applet info
	    String path = GSXML.RESPONSE_ELEM;
	    path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);
	    path = GSPath.appendLink(path, GSXML.APPLET_ELEM);
	    Element app_desc = (Element)doc_.importNode(GSXML.getNodeByPath(mr_response, path), true);
	    // must handle any params that have values that are not 
	    // necessarily known by the service
	    editLocalParams(app_desc, config_.library_name_, collection);
	    page.appendChild(app_desc);

	    // process using the stylesheet
	    Document style_doc = converter_.getDOM(new File(stylesheet));
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return transformer_.transform(style_doc, page);    

	}
	else if (request_type.equals("r")) {
	    // we are processing stuff for the applet send a message to the service, type="query", and take out the something element, and return that as our result - the applet must take xml

	    Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    mr_message.appendChild(mr_request);
	    mr_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	    mr_request.setAttribute(GSXML.TO_ATT, to);
	    
	    // just append the params for now
	    mr_request.appendChild(doc_.importNode(cgi_paramList, true));
	    
	    Element mr_response = (Element)mr_.process(mr_message);
	    System.out.println("applet query response:");
	    // dont need to create a page, just extract the info out.
	    System.out.println(converter_.getString(mr_response));
	    // add in the applet data
	    String path = GSXML.RESPONSE_ELEM;
	    path = GSPath.appendLink(path, GSXML.APPLET_DATA_ELEM);
	    Element applet_info = (Element)GSXML.getNodeByPath(mr_response, path).getFirstChild();
	    System.out.println("applet action: applet data =");
	    System.out.println(converter_.getString(applet_info));
	    return converter_.getString(applet_info);

	}
	else {
	    // should never get here
	    return GSHTML.errorPage("applet action, wrong subaction type");
	}
    }

    /** this method looks through the PARAMs of the applet description for 
     * one with the name 'library'. If its found, it sets the value to be 
     * library_name
     */
    protected void editLocalParams(Element description, String library_name,
				   String full_collection_name) {

	Node child = description.getFirstChild();
	while (child != null) {
	    if (child.getNodeType() == Node.ELEMENT_NODE) {
		String param = child.getNodeName();
		if (param.equals("PARAM")||param.equals("param")) {
		    String name = ((Element)child).getAttribute("NAME");
		    if (name == null || name.equals("")) {
			// somethings wrong!!
		    } else if (name.equals("library")) {
			((Element)child).setAttribute("VALUE", library_name);
		    } else if (name.equals("collection")) {
			((Element)child).setAttribute("VALUE", full_collection_name);
		    }
		    
		}
	    }
	    child = child.getNextSibling();
	}
    }
	
}
