package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.io.File;
public class BrowseAction extends Action {
    

    public String process (Element message) {

	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, message.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.TRANSLATION_ELEM), true));
	// add the system stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	Element request = (Element)message.getElementsByTagName(GSXML.REQUEST_ELEM).item(0);
	
	// the browse type is the subaction
	String browse_type = request.getAttribute(GSXML.SUBACTION_ATT);
	
	if (browse_type.equals("classifier")) {
	    return classifierBrowse(page, request);
	}
	return unknownBrowse(page, request, browse_type);

	
    }
    

    protected String classifierBrowse(Element page, Element request) {

	// check that the stylesheet is present - cant output a page without one. we may adapt this to use unknownquery stylesheet? - or ask for one from the MR
	String stylesheet = GSFile.stylesheetFile(config_, "classifier.xsl");
	if (stylesheet==null) {
	    return GSHTML.errorPage("classifier stylesheet not found!");
	}
	Document style_doc = converter_.getDOM(new File(stylesheet));

	// extract the params from the cgi-request, and check that we have a coll specified
	// first convert short to long names
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	cgi_.toLong(cgi_paramList);
	HashMap params = GSXML.extractParams(cgi_paramList);

	String collection = (String)params.get("collection");
	if (collection == null || collection.equals("")) {
	    return GSHTML.errorPage("classifierbrowse, need to specify a collection!");
	}

	// the first part of the data for the page is the cgi-params
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);

	// get the service info from the MR - this will probably need to be cached somehow later on. and add as a description node to the cgi-request - this doesn't work. change to add the specified value as a current attribute to the param - need to do somthing different for query

	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_info_message.appendChild(mr_info_request);
	mr_info_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);

	String to = collection;
	to = GSPath.appendLink(to, "ClassifierBrowse");
	mr_info_request.setAttribute(GSXML.TO_ATT, to);

	Element mr_info_response = (Element) mr_.process(mr_info_message);
	Element description = doc_.createElement(GSXML.DESCRIPTION_ELEM);

	String path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);
	path = GSPath.appendLink(path, GSXML.CLASSIFIER_ELEM+GSXML.LIST_MODIFIER);
	Node cl = GSXML.getNodeByPath(mr_info_response, path);

	Element imported_classList = (Element)doc_.importNode(cl, true);

	description.appendChild(imported_classList);
	cgi_request.appendChild(description);


	// if there is a cl field in the cgi params, get the classifier info
	String node = (String)params.get("classifier");

	if (node==null|| node.equals("")) {
	    System.out.println("no cl node selected");
	    // if there is no cl set, just output the list
	    GSXSLT.absoluteIncludePaths(style_doc, config_);
	    return transformer_.transform(style_doc, page);       
	}
	
	// get the info for the selected node
	Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_query_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_query_message.appendChild(mr_query_request);

	mr_query_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	mr_query_request.setAttribute(GSXML.TO_ATT, to);

	// paramList - empty for now
	Element paramList = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	mr_query_request.appendChild(paramList);


	// content - contains the classifier node, may be more than one
	// call this resource list for now
	Element query_content = doc_.createElement(GSXML.CONTENT_ELEM);
	mr_query_request.appendChild(query_content);
	
	Element resource_list = doc_.createElement(GSXML.RESOURCE_ELEM+GSXML.LIST_MODIFIER);
	Element resource = doc_.createElement(GSXML.RESOURCE_ELEM);
	resource.setAttribute(GSXML.NAME_ATT, node);
	resource_list.appendChild(resource);
	query_content.appendChild(resource_list);

	Element mr_query_response = (Element)mr_.process(mr_query_message);

	Element response = (Element)GSXML.getChildByTagName(mr_query_response, GSXML.RESPONSE_ELEM);
	Node new_style = GSXML.getChildByTagName(response, "stylesheet");
	if (new_style !=null) {
	    GSXSLT.mergeStylesheets(style_doc, (Element)new_style);
	    response.removeChild(new_style);
	}
	// add the response to the page data
	page.appendChild(doc_.importNode(response, true));

	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return transformer_.transform(style_doc, page);
    }

    protected String unknownBrowse(Element page, Element request, String browse_type) {

	return GSHTML.errorPage("unknown browse subtype: "+browse_type);
    }
}
