package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element;

import java.util.HashMap;
import java.io.File;

public class PageAction extends Action {

    public String process (Element message) {
	
	Element request = (Element)message.getElementsByTagName(GSXML.REQUEST_ELEM).item(0);
	// the page name is the subaction
	String page_name = request.getAttribute(GSXML.SUBACTION_ATT);

	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, message.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from xml_in
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.TRANSLATION_ELEM), true));
	// add the system stuff from xml_in
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	if (page_name.equals("")) {
	    // for now send an error. change to always display the home page?
	    return GSHTML.errorPage(" page action specified, but no page specified");
	}
	if (page_name.equals("home")) {
	    return homePage(page, request);
	} else if (page_name.equals("about")) {
	    return aboutPage(page, request);
	} else {
	    return GSHTML.errorPage("page: "+page_name+" not yet implemented");
	}
    }


    protected String homePage(Element page, Element orig_message) {
	
	// first, get the list of collections from mr
	
	Element coll_list_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	coll_list_message.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	Element coll_list_request = doc_.createElement(GSXML.REQUEST_ELEM);
	coll_list_message.appendChild(coll_list_request);
	coll_list_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	//coll_list_request.setAttribute("info", "collectionList");
	
	Element coll_list_response = (Element)mr_.process(coll_list_message);
	if (coll_list_response==null) {
	    return GSHTML.errorPage("couldn't query the mr about collections");
	}
	
	// second, get the metadata for each collection
	NodeList colls = coll_list_response.getElementsByTagName(GSXML.COLLECTION_ELEM);
	
	//Element metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	//metadata_message.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	//Element metadata_request = doc_.createElement(GSXML.REQUEST_ELEM);
	//metadata_message.appendChild(metadata_request);
	//metadata_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);	
	for (int i=0; i<colls.getLength(); i++) {
	    Element metadata_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    metadata_message.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	    Element metadata_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    metadata_message.appendChild(metadata_request);
	    metadata_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);	
	
	    // get the metadata for each one
	    Element c = (Element)colls.item(i);
	    String name = c.getAttribute(GSXML.NAME_ATT);
	    System.out.println("getting metadata for coll "+name);
	    
	    metadata_request.setAttribute(GSXML.TO_ATT, name); // overwrites the old value
	    System.out.println("metadata message = \n"+converter_.getString(metadata_message));
	    Element metadata_response = (Element)mr_.process(metadata_message);
	    Element coll = (Element)metadata_response.getElementsByTagName(GSXML.COLLECTION_ELEM).item(0);
	    if (coll==null) {
		System.out.println("coll is null, in \n"+converter_.getString(metadata_response));
	    }
	    GSXML.mergeMetadataLists(c, coll); // add the metadata to the original response
	}

	//now the full response is in coll_list_response

	// add it in to the page xml tree
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(coll_list_response, GSXML.RESPONSE_ELEM), true));

	String stylesheet = GSFile.stylesheetFile(config_, "home.xsl");
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	//System.out.println("page=");
	//System.out.println(converter_.getString(page));
	return transformer_.transform(style_doc, page);    
	
    	     
    } // homePage

    protected String aboutPage(Element page, Element request) {

	// extract the params from the cgi-request, 
	// first convert short to long names
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	cgi_.toLong(cgi_paramList);
	HashMap params = GSXML.extractParams(cgi_paramList);
	
	String coll_name = (String)params.get("collection");
	if (coll_name == null || coll_name.equals("")) {
	    coll_name = (String)params.get("serviceCluster"); // cluster name
	}
	if (coll_name == null || coll_name.equals("")) {
	    return GSHTML.errorPage("about page - need to specify coll name or cluster name");
	}

	Element coll_about_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	coll_about_message.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));

	Element coll_about_request = doc_.createElement(GSXML.REQUEST_ELEM);
	coll_about_message.appendChild(coll_about_request);
	coll_about_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	coll_about_request.setAttribute(GSXML.TO_ATT, coll_name);

	Element coll_about_response = (Element)mr_.process(coll_about_message);

	//	System.out.println("coll response=");
	//System.out.println(converter_.getString(coll_about_response));
	// add the response to the page 
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(coll_about_response, GSXML.RESPONSE_ELEM), true));

	// process using the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "about.xsl");
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return transformer_.transform(style_doc, page);    

	
    }
}
