package org.greenstone.gsdl3.util;

import java.util.HashMap;
import org.w3c.dom.Element;

public class CGIArgConverter {

    protected HashMap long_map_=null;
    protected HashMap short_map_=null;
    
    public CGIArgConverter() {
	long_map_ = new HashMap();
	short_map_ = new HashMap();

	// initialize with the ones we know about
	long_map_.put("case", "k");
	short_map_.put("k", "case");

	long_map_.put("stem", "s");
	short_map_.put("s", "stem");
	
	long_map_.put("queryLevel", "ql");
	short_map_.put("ql", "queryLevel");
	
	long_map_.put("index", "i");
	short_map_.put("i", "index");
	
	long_map_.put("matchMode", "mm");
	short_map_.put("mm", "matchMode");

	long_map_.put("sortBy", "sb");
	short_map_.put("sb", "sortBy");
	
	long_map_.put("maxDocs", "md");
	short_map_.put("md", "maxDocs");

	short_map_.put("c", "collection"); 
	long_map_.put("collection", "c");

	short_map_.put("a", "action"); 
	long_map_.put("action", "a");

	short_map_.put("sa", "subaction"); 
	long_map_.put("subaction", "sa");	

	short_map_.put("sn", "serviceName"); 
	long_map_.put("serviceName", "sn");	

	short_map_.put("sc", "serviceCluster");
	long_map_.put("serviceCluster", "sc");

	short_map_.put("l", "lang"); 
	long_map_.put("lang", "l");

	long_map_.put("query", "q");
	short_map_.put("q", "query");
	
	long_map_.put("resource", "r");
	short_map_.put("r", "resource");

	short_map_.put("cl", "classifier"); 
	long_map_.put("classifier", "cl");
    }

    public boolean toLong(Element param_list) {
	
	if (!param_list.getNodeName().equals("paramList")) {
	    System.out.println("wrong element to CGIArgConverter:toLong");
	    return false;
	}
	
	Element param = (Element)param_list.getFirstChild();
	while (param!=null) {
	    String name = param.getAttribute("name");
	    param.setAttribute("name", toLong(name));
	    param = (Element)param.getNextSibling();
	}
	return true;

    }
    
    public boolean  addShortNames(Element param_list) {
	if (!param_list.getNodeName().equals("paramList")) {
	    System.out.println("wrong element to CGIArgConverter:toShort");
	    return false;
	}
	
	Element param = (Element)param_list.getFirstChild();
	while (param!=null) {
	    String name = param.getAttribute("name");
	    param.setAttribute("shortname", toShort(name));
	    param = (Element)param.getNextSibling();
	}
	return true;

    }
    

    public String toLong(String short_name) {
	
	
	if (short_map_.containsKey(short_name)) {
	    return (String)short_map_.get(short_name);
	}
	System.err.println("CGIArgConverter: short name '"+short_name+"' not found in map!");
	return short_name; // just return the original
    }

    public String toShort(String long_name) {

	if (long_map_.containsKey(long_name)) {
	    return (String)long_map_.get(long_name);
	}

	// else need to create a new mapping
	String short_name = newShortName(long_name);
	long_map_.put(long_name, short_name);
	short_map_.put(short_name, long_name);
	return short_name;
    }

    protected String newShortName(String long_name) {

	// for now, try the first letter, then first two etc.
	int i=1;
	String short_try= long_name.substring(0,1);
	while(short_map_.containsKey(short_try)&& i<long_name.length()) {
	    i++;
	    short_try = long_name.substring(0, i);
	}
	if (i==long_name.length()) {
	    return ""; // couldn't find one
	}
	return short_try;
    }
}


